#ifndef		CALCULATOR_H
#define		CALCULATOR_H


/**************************************************************
*
*	Fisiere header
*
**************************************************************/
#include	"main.h"


/**************************************************************
*
*	Definitii macro
*
**************************************************************/
#ifdef ALLOCATE
	#define  GLOBAL
#else
	#define  GLOBAL  extern
#endif

#define		TRUE	1
#define		FALSE	0


//#define TRACE_LEX printf("token is %s\n", yytext);
#define TRACE_LEX

//#define msg1(a)
//#define msg2(a,b)
//#define msg3(a,b,c)
//#define msg1 printf
//#define msg2 printf
//#define msg3 printf

/**************************************************************
*
*	Definitii structuri
*
**************************************************************/

// structira pentru cuvinte rezervate
typedef struct
	{
	char*		word;
	long		token;
	}								ResWordElem;

// specificatii pentru variabile
typedef enum
	{
	TipNon,
	TipByVal,TipByRef
	}										TipParamSpec;


#pragma pack(1)
// structura pentru lista de variabile
typedef	 struct	_VarList
	{
	TipParamSpec		TipSpec;
	int					Tip;
	char*				Nume;
	struct _VarList*	pNext;
	union
		{
		long			Int;
		double			Float;
		char*			String;
		int				Boolean;
		};
	}										VarList;

//
//		Structuri noduri folosite penrtu expresii numerice
//


// tipuri de noduri
typedef enum
	{
	TipNodOpr,TipNodVar,
	TipNodCtInt,TipNodCtFloat,
	TipNodNumFnc
	}										TipNumNod;

// nod generic
typedef struct _GenNumNode
	{
	TipNumNod			Tip;
	}										GenNumNode;

// nod pentru constanta de tip intreg
typedef struct _CtIntNode
	{
	TipNumNod			Tip;
	long				Val;
	}										CtIntNode;

// nod pentru constanta de tip float
typedef struct _CtFloatNode
	{
	TipNumNod			Tip;
	double				Val;
	}										CtFloatNode;

// nod pentru variabila de tip intreg sau float
typedef struct _VarNumNode
	{
	TipNumNod				Tip;
	VarList*				pVar;
	}										VarNumNode;

// nod pentru operator numeric
typedef struct _OprNumNode
	{
	TipNumNod			Tip;
	int					Opr;
	int					NrOpd;
	GenNumNode			*NodList[2];
	}										OprNumNode;


// tipuri pentru functia EvalExprNumTree
typedef enum
	{
	TipInt,TipFloat
	}										TipNum;
typedef struct _EvalNumStruct
	{
	TipNum				Tip;
	union
		{
		long		Int;
		double		Float;
		};
	}
								EvalNumStruct;


//
//		Structuri noduri folosite penrtu expresii booleene
//

// tipuri de noduri
typedef enum
	{
	TipNodOprBool,TipNodVarBool,
	TipNodCtBool,TipNodCmpBool,
	TipNodBoolFnc
	}										TipBoolNod;

// nod generic
typedef struct _GenBoolNode
	{
	TipBoolNod			Tip;
	}										GenBoolNode;
// nod pentru constanta de tip boolean
typedef struct _CtBoolNode
	{
	TipBoolNod			Tip;
	char				Val;
	}										CtBoolNode;
// nod pentru variabila de tip boolean
typedef struct _VarBoolNode
	{
	TipBoolNod				Tip;
	VarList*				pVar;
	}										VarBoolNode;
// nod pentru operator boolean
typedef struct _OprBoolNode
	{
	TipBoolNod			Tip;
	int					Opr;
	int					NrOpd;
	GenBoolNode			*NodList[1];
	}										OprBoolNode;
// nod pentru operator ce compara doua expresii numerice
typedef struct _CmpBoolNode
	{
	TipBoolNod			Tip;
	int					Opr;
	GenNumNode			*NodList[2];
	}										CmpBoolNode;


//
//		Structuri noduri folosite pentru expresii string
//

// tipuri de noduri
typedef enum
	{
	TipNodOprStr,TipNodVarStr,
	TipNodCtStr,TipNodStrFnc
	}										TipStrNod;

// nod generic
typedef struct _GenStrNode
	{
	TipStrNod			Tip;
	}										GenStrNode;

// nod pentru constanta de tip string
typedef struct _CtStrNode
	{
	TipStrNod			Tip;
	char*				Val;
	}										CtStrNode;
// nod pentru variabila de tip string
typedef struct _VarStrNode
	{
	TipStrNod				Tip;
	VarList*				pVar;
	}										VarStrNode;
// nod pentru operator string
typedef struct _OprStrNode
	{
	TipStrNod			Tip;
	int					Opr;
	int					NrOpd;
	GenStrNode*			NodList[2];
	}										OprStrNode;


//
//		Structuri element de lista pentru lista de intructiuni
//

// tipuri de elemente
typedef enum
	{
	TipInstAttribNum,TipInstAttribBool,
	TipInstAttribStr,
	TipInstIfThenElse,TipInstWhile,
	TipInstRepeat, TipInstForToNext,
	TipInstBreak,TipInstExit,
	TipFncCall
	}											TipElemInst;

// element intructiune generica
typedef struct _GenInstElem
	{
	TipElemInst				Tip;
	struct _GenInstElem*	pNext;
	}										GenInstElem;

// element intructiune atribuire numerica
typedef struct _AssignNumInstElem
	{
	TipElemInst				Tip;
	struct _GenInstElem*	pNext;
	VarList*				pVar;
	GenNumNode*				pNumTree;
	}										AssignNumInstElem;

// element intructiune atribuire booleana
typedef struct _AssignBoolInstElem
	{
	TipElemInst				Tip;
	struct _GenInstElem*	pNext;
	VarList*				pVar;
	GenBoolNode*			pBoolTree;
	}										AssignBoolInstElem;

// element intructiune atribuire string
typedef struct _AssignStrInstElem
	{
	TipElemInst				Tip;
	struct _GenInstElem*	pNext;
	VarList*				pVar;
	GenStrNode*				pStrTree;
	}										AssignStrInstElem;

// element folosit pentru list elseif din intructiunea 'if then else'
typedef struct _ElseIfCell
	{
	GenBoolNode*			pElseIfTree;
	GenInstElem*			pElseIfDoList;
	struct _ElseIfCell*		pNext;
	}										ElseIfCell;

//	element intructiune 'if then elseif ... else endif'
//	Daca nu exista ramura Else , atunci pElseList e NULL
typedef struct _IfThenElseInstElem
	{
	TipElemInst				Tip;
	struct _GenInstElem*	pNext;
	GenBoolNode*			pIfTree;
	GenInstElem*			pIfDoList;
	ElseIfCell*				pElseIfList;
	GenInstElem*			pElseDoList;
	}										IfThenElseInstElem;
//	element intructiune 'while do endwhile'
typedef struct _WhileInstElem
	{
	TipElemInst				Tip;
	struct _GenInstElem*	pNext;
	GenBoolNode*			pWhileTree;
	GenInstElem*			pDoList;
	}										WhileInstElem;
//	element intructiune 'repeat until endrepeat'
typedef struct _RepeatInstElem
	{
	TipElemInst				Tip;
	struct _GenInstElem*	pNext;
	GenBoolNode*			pRepeatTree;
	GenInstElem*			pDoList;
	}										RepeatInstElem;

//	element intructiune 'for to step next'
typedef struct _ForToNextInstElem
	{
	TipElemInst				Tip;
	struct _GenInstElem*	pNext;
	VarList*				pVar;
	GenNumNode*				pBeginValue;
	GenNumNode*				pEndValue;
	GenNumNode*				pStepValue;
	GenInstElem*			pDoList;
	}									ForToNextInstElem;

//	element intructiune break
typedef struct _BreakInstElem
	{
	TipElemInst				Tip;
	struct _GenInstElem*	pNext;
	}										BreakInstElem;

//	element intructiune exit
typedef struct _ExitInstElem
	{
	TipElemInst				Tip;
	struct _GenInstElem*	pNext;
	}										ExitInstElem;


typedef struct _Program
	{
	VarList*			pVarList;
	VarList*			pParamList;
	GenInstElem*		pInstrList;
	long				LineNo;
	char*				strPrgName;
	struct _Program*	pCaller;
	}											Program;


#pragma pack()
/***************************************************************
*
*	Variabile globale
*
***************************************************************/
extern	char* yytext;
extern	ResWordElem ResWordList[];

GLOBAL		Program*		pCurrentProgram;



/***************************************************************
*
*	Definitii de functii
*
***************************************************************/

/*-------------------------------------------------------------
*	Definite de lex & yacc
*-------------------------------------------------------------*/

int		yyparse(void*);
int		yyerror( char * s);

/*-------------------------------------------------------------
*	Functii definite in Calc.c
*-------------------------------------------------------------*/


/*-------------------------------------------------------------
*	Functii yacc
*-------------------------------------------------------------*/
// adauga o variabila la lista de variabile
void
	AddToVarList	(
					Program*		pPrg,
					char*			Name,
					int				Tip
					);
// adauga un parametru la lista de parametrii
void
	AddToParamList	(
					Program*		pPrg,
					char*			Name,
					int				Tip,
					TipParamSpec	TipSpec
					);
//	Dezalocarea listei de variabile
void
	FreeVarList(Program*	pPrg);
//	Dezalocarea listei de parametrii
void
	FreeParamList(Program*	pPrg);

// creaza un nod de tip operator numeric cu n operanzi
GenNumNode*
	NewOprNode		(
					long		TipOpr,
					long		NrOpd,
					...
					);
// creaza un nod de tip constanta intreaga
GenNumNode*
	NewCtIntNode(
				long		ValOpd
				);
// creaza un nod de tip constanta float
GenNumNode*
	NewCtFloatNode	(
					double		ValOpd
					);
// creaza un nod de tip variabila numerica
GenNumNode*
	NewIdNode	(
				VarList*		pIdOpd
				);
// evalueaza un arbore de tip expresie numerica
EvalNumStruct
	EvalExprNumTree(GenNumNode*		pNumTree);
// Face conversia de la float la intreg
EvalNumStruct
	ConvertFloatToInt(EvalNumStruct  NumOpd);
// Face conversia de la intreg la float
EvalNumStruct
	ConvertIntToFloat(EvalNumStruct  NumOpd);


// Dezaloca un arbore numeric
void
	FreeNumTree( GenNumNode* pNumTree );


// creaza un nod de tip operator boolean cu n operanzi
GenBoolNode*
	NewBoolOprNode	(
					long		TipOpr,
					long		NrOpd,
					...
					);
// creaza un nod de tip comparatie intre doua expresii numerice
GenBoolNode*
	NewBoolCmpNode	(
					long			TipOpr,
					GenNumNode*		pFirst,
					GenNumNode*		pSecond
					);
// creaza un nod de tip constanta booleana
GenBoolNode*
	NewCtBoolNode	(
					char		ValOpd
					);
// creaza un nod de tip variabila booleana
GenBoolNode*
	NewIdBoolNode	(
					VarList*		pIdOpd
					);

// evalueaza un arbore de tip expresie booleana
char
	EvalExprBoolTree(GenBoolNode*		pBoolTree);

// Dezaloca un arbore boolean
void
	FreeBoolTree( GenBoolNode* pBoolTree );

// creaza un nod de tip constanta string
GenStrNode*
	NewCtStrNode	(
					char*		ValOpd
					);

// creaza un nod de tip variabila string
GenStrNode*
	NewIdStrNode	(
					VarList*		pIdOpd
					);

// creaza un nod de tip operator string cu n operanzi
GenStrNode*
	NewStrOprNode	(
					long		TipOpr,
					long		NrOpd,
					...
					);

// evalueaza un arbore de tip expresie string
char*
	EvalExprStrTree(GenStrNode*		pStrTree);
// dezaloca un arbore de tip expresie string
void
	FreeStrTree(GenStrNode*		pStrTree);



// creaza un element de tip intructiune assignare numerica
GenInstElem*
	NewAssignNumInst(
					VarList*			pNumId,
					GenNumNode*			pNumTree
					);
// creaza un element de tip intructiune assignare booleana
GenInstElem*
	NewAssignBoolInst(
					VarList*			pBoolId,
					GenBoolNode*		pBoolTree
					);
// creaza un element de tip intructiune assignare string
GenInstElem*
	NewAssignStrInst(
					VarList*			pStrId,
					GenStrNode*			pStrTree
					);
// creaza un element de tip intructiune while
GenInstElem*
	NewWhileInst	(
					GenBoolNode*		pBoolTree,
					GenInstElem*		pDoList
					);
// creaza un element de tip intructiune repeat
GenInstElem*
	NewRepeatInst	(
					GenBoolNode*		pBoolTree,
					GenInstElem*		pDoList
					);
// creaza un element de tip intructiune 'for to step next'
GenInstElem*
	NewForToNextInst(
					VarList*				pVar,
					GenNumNode*				pBeginValue,
					GenNumNode*				pEndValue,
					GenNumNode*				pStepValue,
					GenInstElem*			pDoList
					);
// adauga un element la sfarsitul listei de intructiuni
GenInstElem*
	AddInstToList	(
					GenInstElem*		pInstList,
					GenInstElem*		pInstElem
					);
// creaza un element elseif pentru intructiunea if then else
ElseIfCell*
	NewElseIfCell	(
					GenBoolNode*		pElseIfTree,
					GenInstElem*		pElseIfDoList
					);
// adauga un element la sfarsitul listei elseif
ElseIfCell*
	AddElseIfToList	(
					ElseIfCell*		pElseIfList,
					ElseIfCell*		pElseIfElem
					);
// creaza un element de tip intructiune if then else
GenInstElem*
	NewIfThenElseInst	(
						GenBoolNode*		pIfBoolTree,
						GenInstElem*		pIdDoList,
						ElseIfCell*			pElseIfList,
						GenInstElem*		pElseDoList
						);
// creaza un element de tip intructiune break
GenInstElem*
	NewBreakInst	( void );
// creaza un element de tip intructiune exit
GenInstElem*
	NewExitInst		( void );

// executa lista de intructiuni
void
	ExecuteInstrList	(
						GenInstElem*		pInstrList
						);
// elibereaza memoria alocata de lista de intructiuni
void
	FreeInstrList	(
					GenInstElem*		pInstrList
					);

// alocarea unei structuri de tip program
Program*
	NewProgram	(
				char*		strPrgName
				);
// executa programul
void
	ExecuteProgram		(
						Program*			pPrg
						);
// elibereaza memoria alocata de lista de intructiuni
void
	FreeProgram		(
					Program*		pPrg
					);

/*-------------------------------------------------------------
*	Functii lex
*-------------------------------------------------------------*/

// token STRING
char*
	lex_token_string_concat( char* strCt1, char* strCt2 );
//	cautare cuvinte rezervate in lista
int
	lex_token_res_word( char*	ResWord, int size);
// functie lex , se aloca un buffer
void*
	lex_set_buffer		(
						void*		buffer
						);
// functie lex , se forteaza scanarea unui buffer
void
	lex_switch_buffer	(
						void*		buffer
						);
// functie lex , se elibereaza resursele folosite pentru scanarea buffer-ului
void
	lex_free_buffer		(
						void*		buffer_state
						);




#endif
