
%{
#include "main.h"

// token ID, ID_INT, ID_FLOAT , ID_SRING , ID_BOOL
int lex_token_id	( Program* pPrg,YYSTYPE* yylval,char* strToken );
// token FLOAT
int lex_token_float	( YYSTYPE* yylval,char* strToken );
// token INT
int	lex_token_int	( YYSTYPE* yylval,char* strToken );

ResWordElem ResWordList[]=
{
	/* declaratie program			*/
	{"program",			PROGRAM		},
	{"endprogram",		ENDPROGRAM	},
	{"byref",			BYREF		},
	{"byval",			BYVAL		},

	/* declaratii de variabile		*/
	{"dim",				DIM			},
	{"as",				AS			},

	/* if then else					*/
	{"if",				IF			},
	{"then",			THEN		},
	{"elseif",			ELSEIF		},
	{"else",			ELSE		},
	{"endif",			ENDIF		},

	/* while do endwhile			*/
	{"while",			WHILE		},
	{"do",				DO			},
	{"endwhile",		ENDWHILE	},

	/* repeat until endrepeat		*/
	{"repeat",			REPEAT		},
	{"until",			UNTIL		},
	{"endrepeat",		ENDREPEAT	},

	/* for to step next				*/
	{"for",				FOR			},
	{"to",				TO			},
	{"step",			STEP		},
	{"next",			NEXT		},

	/* break , exit					*/
	{"break",			BREAK		},
	{"exit",			EXIT		},

	/* tipuri de date				*/
	{"integer",			TYPE_INT	},
	{"float",			TYPE_FLOAT	},
	{"string",			TYPE_STRING	},
	{"boolean",			TYPE_BOOL	},

	/* operatori booleeni			*/
	{"and",				AND			},
	{"or",				OR			},
	{"not",				NOT			},

	/* constante booleeni			*/
	{"true",			CT_TRUE		},
	{"false",			CT_FALSE	},

	/* functii	void						*/
	{"Print",					PRINT		},
	{"ExecProgram",				EXECPROGRAM	},

	/* functii	numerice					*/
	{"FncNum1",			FNC_NUM_1	},
	{"FncNum2",			FNC_NUM_2	},

	/* functii	booleene					*/
	{"FncBool1",		FNC_BOOL_1	},
	{"FncBool2",		FNC_BOOL_2	},
	{"FncBool3",		FNC_BOOL_3	},
	{"FncBool4",		FNC_BOOL_4	},
	{"FncBool5",		FNC_BOOL_5	},
	{"StrCmp",			STRCMP		},

	/* functii	string						*/
	{"Mid",						MID			},
};

#ifndef VSIZE
	#define VSIZE(v) (sizeof(v)/sizeof(*(v)))
#endif

#define	YY_DECL  int yylex( YYSTYPE* yylval, void* _pPrg)

#define	pPrg	((Program*)_pPrg)
%}

%option		noyywrap
%x string


letter	[a-zA-Z_]
digit	[0-9]

%%

	/*----------------------------------------------.
	|			eliminare space si tab				|
	`----------------------------------------------*/
[ \t]+			;

	/*----------------------------------------------.
	|			Comentarii  stil C++				|
	`----------------------------------------------*/
"//"[^\n]*		;

	/*----------------------------------------------.
	|				new line						|
	`----------------------------------------------*/
\r\n	pPrg->LineNo++;  return NL;
\n		pPrg->LineNo++;  return NL;
\r		pPrg->LineNo++;  return NL;

	/*----------------------------------------------.
	|	cuvinte rezervate si identificatori			|
	`----------------------------------------------*/
{letter}({letter}|{digit})*		{

	int			i;
	TRACE_LEX
	if (i=lex_token_res_word(yytext,VSIZE(ResWordList)))
		return i;
	return lex_token_id(pPrg,yylval,yytext);
	}

	/*----------------------------------------------.
	|				constante float					|
	`----------------------------------------------*/
{digit}+\.{digit}+			return lex_token_float(yylval, yytext );

	/*----------------------------------------------.
	|				constante intregi				|
	`----------------------------------------------*/
{digit}+					return lex_token_int(yylval, yytext );

	/*----------------------------------------------.
	|				operatori aritmetici			|
	`----------------------------------------------*/
"+"				return ADD;
"-"				return SUB;
"*"				return MUL;
"/"				return DIV;
":="			return ASSIGN;

	/*----------------------------------------------.
	|				comparatii numerice				|
	`----------------------------------------------*/
"<"				return LT;
"<="			return LE;
">"				return GT;
">="			return GE;
"=="			return EQ;
"<>"			return NE;

	/*----------------------------------------------.
	|				operatori string				|
	`----------------------------------------------*/
"&"				return CONCAT;

	/*----------------------------------------------.
	|		token care intoarce codul ASCII			|
	`----------------------------------------------*/
"("				return yytext[0];
")"				return yytext[0];
","				return yytext[0];
"%"				return yytext[0];


	/*----------------------------------------------.
	|		recunoastere string-uri					|
	`----------------------------------------------*/
\"		BEGIN(string);yylval->StringVal = strdup("");
<string>
{
[\"]	BEGIN(INITIAL);return CT_STRING;
\"\"	yylval->StringVal = lex_token_string_concat(yylval->StringVal,"\"");
"\\n"	yylval->StringVal = lex_token_string_concat(yylval->StringVal,"\n");
"\\r"	yylval->StringVal = lex_token_string_concat(yylval->StringVal,"\r");
"\\t"	yylval->StringVal = lex_token_string_concat(yylval->StringVal,"\t");
"\\"	yylval->StringVal = lex_token_string_concat(yylval->StringVal,"\\");
[^"\n\\]*	yylval->StringVal = lex_token_string_concat(yylval->StringVal,yytext);
\n		BEGIN(INITIAL);yyerror("bad string format \n");	return 0;
<<EOF>>	BEGIN(INITIAL);yyerror("bad string format \n");	return 0;
}

	/*----------------------------------------------.
	|		token care intoarce codul ASCII			|
	`----------------------------------------------*/
.				return yytext[0];


%%

// functie lex , se aloca un buffer
void*
	lex_set_buffer		(
						void*		buffer
						)
	{
	return (void*)yy_scan_string((char*)buffer);
	}


// functie lex , se forteaza scanarea unui buffer
void
	lex_switch_buffer	(
						void*		buffer_state
						)
	{
	yy_switch_to_buffer((YY_BUFFER_STATE)buffer_state);
	}


// functie lex , se elibereaza resursele folosite pentru scanarea buffer-ului
void
	lex_free_buffer		(
						void*		buffer_state
						)
	{
	yy_delete_buffer((YY_BUFFER_STATE)buffer_state);
	}
