#ifndef		FUNCTIONS_H
#define		FUNCTIONS_H

/**************************************************************
*
*	Fisiere header
*
**************************************************************/
#include	"main.h"



/**************************************************************
*
*	Definitii structuri
*
**************************************************************/

//
//	Structuri elemente folosite pentru parametrii de functii
//
// tipuri de parametrii
typedef enum
	{
	TipParamExprNum,TipParamExprBool,
	TipParamExprStr,
	TipParamIdInt,TipParamIdFloat,
	TipParamIdBool,TipParamIdString
	}										TipParam;

typedef struct _Param
	{
	TipParam			Tip;
	struct _Param*		pNext;
	union
		{
		GenNumNode*			pNumTree;
		GenBoolNode*		pBoolTree;
		GenStrNode*			pStrTree;
		VarList*			pIntVar;
		VarList*			pFloatVar;
		VarList*			pStrVar;
		VarList*			pBoolVar;
		};
	}										Param;

//
//	Structuri elemente folosite pentru functii
//

// tipuri de noduri
typedef enum
	{
	TipPrintFnc, TipExecPrgFnc,
	TipFncNum1Fnc, TipFncNum2Fnc,
	TipFncBool1Fnc, TipFncBool2Fnc,
	TipFncBool3Fnc,TipFncBool4Fnc,
	TipFncBool5Fnc,TipStrcmpFnc,
	TipFncStr1Fnc
	}										TipFncElem;

// nod generic
typedef struct _GenFncElem
	{
	TipFncElem			Tip;
	}										GenFncElem;

// nod functie print
typedef struct _PrintFncElem
	{
	TipFncElem			Tip;
	GenStrNode*			pParam1;
	Param*				pParam2;
	}										PrintFncElem;

// nod functie print
typedef struct _ExecPrgFncElem
	{
	TipFncElem			Tip;
	char*				PrgName;
	Param*				pParamList;
	Program*			pPrg;
	}										ExecPrgFncElem;

// nod functie ZileLucratoare
typedef struct _Fnc1FncElem
	{
	TipFncElem			Tip;
	GenNumNode*			pAn;
	GenNumNode*			pLuna;
	}										FncNum1FncElem;

// nod functie GrilaImpozitare
typedef struct _FncNum2FncElem
	{
	TipFncElem			Tip;
	GenNumNode*			pSuma;
	}										FncNum2FncElem;


// nod functie AngajatCC
typedef struct _FncBool1FncElem
	{
	TipFncElem			Tip;
	GenNumNode*			pAngajat;
	GenNumNode*			pFirma;
	GenNumNode*			pAn;
	GenNumNode*			pLuna;
	VarList*			pOL;
	VarList*			pOS;
	VarList*			pOCO;
	VarList*			pOP;
	VarList*			pPB;
	VarList*			pPP;
	}										FncBool1FncElem;

// nod functie AngajatNegociere
typedef struct _FncBool2FncElem
	{
	TipFncElem			Tip;
	GenNumNode*			pAngajat;
	GenNumNode*			pFirma;
	GenNumNode*			pAn;
	GenNumNode*			pLuna;
	VarList*			pSB;
	VarList*			pSN;
	VarList*			pValuta;
	}										FncBool2FncElem;


// nod functie DeducereGenerala
typedef struct _FncBool3FncElem
	{
	TipFncElem			Tip;
	GenNumNode*			pAn;
	GenNumNode*			pLuna;
	VarList*			pDPB;
	VarList*			pCCP;
	VarList*			pPlafCAS;
	}										FncBool3FncElem;

// nod functie DeducereSuplimentara
typedef struct _FncBool4FncElem
	{
	TipFncElem			Tip;
	GenNumNode*			pAngajat;
	GenNumNode*			pAn;
	GenNumNode*			pLuna;
	VarList*			pSCDED;
	}										FncBool4FncElem;

// nod functie AngajatInfoInacdrare
typedef struct _FncBool5FncElem
	{
	TipFncElem			Tip;
	GenNumNode*			pAngajat;
	GenNumNode*			pAn;
	GenNumNode*			pLuna;
	VarList*			pTipContract;
	VarList*			pCondLucru;
	VarList*			pCoefCondLucru;
	}										FncBool5FncElem;

// nod functie Strcmp
typedef struct _StrcmpFncElem
	{
	TipFncElem			Tip;
	GenStrNode*			pStr1;
	GenStrNode*			pStr2;
	}										StrcmpFncElem;


// nod functie VB mid
typedef struct _FncStr1FncElem
	{
	TipFncElem			Tip;
	GenStrNode*			pStr;
	GenNumNode*			pFirst;
	GenNumNode*			pLength;
	}										FncStr1FncElem;

//	element intructiune apel functie
typedef struct _FncCallElem
	{
	TipElemInst				Tip;
	struct _GenInstElem*	pNext;
	GenFncElem*				pFnc;
	}										FncCallElem;

// nod "expr_num" pentru functie cu rezultat numeric
typedef struct _FncNumNode
	{
	TipNumNod			Tip;
	GenFncElem*			pNumFnc;
	}										FncNumNode;

// nod "expr_bool" pentru functie cu rezultat boolean
typedef struct _FncBoolNode
	{
	TipBoolNod			Tip;
	GenFncElem*			pBoolFnc;
	}										FncBoolNode;

// nod "expr_str" pentru functie cu rezultat string
typedef struct _FncStrNode
	{
	TipStrNod			Tip;
	GenFncElem*			pStrFnc;
	}										FncStrNode;

/***************************************************************
*
*	Definitii de functii
*
***************************************************************/
// creaza un element de tip parametru numeric
Param*
	NewParamExprNum	(
					GenNumNode*			pNumNode
					);

// creaza un element de tip parametru boolean
Param*
	NewParamExprBool(
					GenBoolNode*		pBoolNode
					);

// creaza un element de tip parametru string
Param*
	NewParamExprStr	(
					GenStrNode*			pStrNode
					);
// creaza un element de tip parametru variabila int
Param*
	NewParamIdInt	(
					VarList*		pIntVar
					);
// creaza un element de tip parametru variabila float
Param*
	NewParamIdFloat	(
					VarList*		pFloatVar
					);
// creaza un element de tip parametru variabila string
Param*
	NewParamIdStr	(
					VarList*		pStrVar
					);
// creaza un element de tip parametru variabila booleana
Param*
	NewParamIdBool	(
					VarList*		pBoolVar
					);
// adauga un parametru la lista de parametrii
Param*
	AddParamToParamList	(
						Param*		pParamList,
						Param*		pParam
						);
// creaza un nod expresie numerica de tip functie numerica
GenNumNode*
	NewNumFncNode	(
					GenFncElem*		pNumFnc
					);
// creaza un nod expresie booleana de tip functie booleana
GenBoolNode*
	NewBoolFncNode	(
					GenFncElem*		pBoolFnc
					);
// creaza un nod "expr_str" de tip functie string
GenStrNode*
	NewStrFncNode	(
					GenFncElem*		pStrFnc
					);

// creaza un element de tip functie "print"
GenFncElem*
	NewPrintFncElem	(
					GenStrNode*			pParam1,
					Param*				pParam2
					);
// creaza un element de tip functie "ExecProgram"
GenFncElem*
	NewExecPrgElem	(
					char*			PrgName,
					Param*			pParamList
					);
// creaza un element de tip functie "ZileLucratoare"
GenFncElem*
	NewFncNum1Elem	(
					GenNumNode*		pAn,
					GenNumNode*		pLuna
					);
// creaza un element de tip functie "GrilaImpozit"
GenFncElem*
	NewFncNum2Elem	(
					GenNumNode*		pSuma
					);
// creaza un element de tip functie "AngajatCC"
GenFncElem*
	NewFncBool1Elem	(
					GenNumNode*		pAngajat,
					GenNumNode*		pFirma,
					GenNumNode*		pAn,
					GenNumNode*		pLuna,
					VarList*		pOL,
					VarList*		pOS,
					VarList*		pOCO,
					VarList*		pOP,
					VarList*		pPB,
					VarList*		pPP
					);
// creaza un element de tip functie "AngajatNegociere"
GenFncElem*
	NewFncBool2Elem	(
					GenNumNode*		pAngajat,
					GenNumNode*		pFirma,
					GenNumNode*		pAn,
					GenNumNode*		pLuna,
					VarList*		pSB,
					VarList*		pSN,
					VarList*		pValuta
					);
// creaza un element de tip functie "DeducereGenerala"
GenFncElem*
	NewFncBool3Elem	(
					GenNumNode*		pAn,
					GenNumNode*		pLuna,
					VarList*		pDPB,
					VarList*		pCCP,
					VarList*		pPlafCAS
					);

// creaza un element de tip functie "DeducereSuplimentara"
GenFncElem*
	NewFncBool4Elem	(
					GenNumNode*		pAngajat,
					GenNumNode*		pAn,
					GenNumNode*		pLuna,
					VarList*		pSCDED
					);
// creaza un element de tip functie "AngajatInfoIncadrare"
GenFncElem*
	NewFncBool5Elem	(
					GenNumNode*		pAngajat,
					GenNumNode*		pAn,
					GenNumNode*		pLuna,
					VarList*		pTipContract,
					VarList*		pCondLucru,
					VarList*		pCoefCondLucru
					);
// creaza un element de tip functie "Strcmp"
GenFncElem*
	NewFncStrcmpElem(
					GenStrNode*		pStr1,
					GenStrNode*		pStr2
					);
// creaza un element de tip functie VB  mid
GenFncElem*
	NewFncStr1Elem	(
					GenStrNode*		pStr,
					GenNumNode*		pFirst,
					GenNumNode*		pLength
					);
// creaza un element de tip apel functie
GenInstElem*
	NewFncCallInst	(
					GenFncElem*				pFnc
					);

// executa functii cu rezultat numeric
EvalNumStruct
	ExecNumFnc		(
					GenFncElem*		pFnc
					);
// executa functii cu rezultat boolean
char
	ExecBoolFncNode	(
					GenFncElem*		pNumFnc
					);
// executa functii cu rezultat string
char*
	ExecStrFncNode	(
					GenFncElem*		pStrFnc
					);


// functie de executie pt. elemente de tip functie
void
	ExecuteFncCallInstr	(
						GenInstElem*		pInstr
						);
// elibereaza memoria alocata de intructiunea de tip functie
void
	FreeFncCallInstr	(
						GenInstElem*		pInstr
						);

// elibereaza memoria alocata de un nod numeric de tip functie
// functii de intorc valoare numerica
void
	FreeNumFncNode		(
						GenFncElem*		pNumFnc
						);
// elibereaza memoria alocata de un nod boolean de tip functie
// functii de intorc valoare booleana
void
	FreeBoolFncNode		(
						GenFncElem*			pBoolFnc
						);
// elibereaza memoria alocata "expr_str" de tip functie
// functii ce intorc valoare srting
void
	FreeStrFncNode		(
						GenFncElem*			pStrFnc
						);

// verifica corespondenta intre parametrii formali si cei actuali
bool
	VerifyProgramParams	(
						ExecPrgFncElem*		pPrgFnc
						);
// transmite valorile parametrilor actuali catre cei formali
void
	SetProgramParams	(
						ExecPrgFncElem*		pPrgFnc
						);
// transmite valorile parametrilor formali catre cei actuali
void
	GetProgramParams	(
						ExecPrgFncElem*		pPrgFnc
						);

#endif
